/******************************************************************/
/* filename :       vm_postdraws.g                                */
/* Description:     Estimating the paramaters of a BVAR           */
/*                                                                */
/******************************************************************/

new;
closeall; 
cls;

#include vb_spec.g; 
#include pathspec.g;

_verbose = 1;    /* set it equal to 1 for detailed printing of code outputs */
mhrun      = "1";  /* 1 = posterior draws  */
nobsscale_ = 1;  /* create multiple of the sample size; nobsscale_> 100 
                    means that parameters are fixed at OLS values */

/******************************************************** 
* Import data series                                    *
********************************************************/

#include vb_loaddata.g;
k       =   1 + nvar_*nlags_;

/******************************************************** 
* Initialize directories, etc...                        *
********************************************************/

parapath = parapath $+ "\\" $+ "\mhrun" $+ mhrun;
chkdir = chk_dir(parapath);

/** output file name 
**/
oparavar = parapath $+ "\\paravar.dat";

/** create output file 
**/
numcols = nvar_*k + nvar_; 
create fhparavar = ^oparavar with PARA, numcols, 8;  

/* OLS ESTIMATORS
*/
phi_tilde   =   (XX'*YY)/(XX'*XX) ;
sigma_tilde =   (1/nobs)*((YY'*YY)-(XX'*YY)'( (XX'*YY)/(XX'*XX) ));

/******************************************************** 
** Direct Sampling
********************************************************/

nblocks     =    100;
nsim        =    1000;
nsim_total  =    nblocks*nsim;

phi_mean      =   zeros(1,nvar_*k);
sigmatr_mean  =   zeros(1,nvar_^2);

iblock      =    1;

do until iblock > nblocks;
   
    if _verbose;
        "";
        "----------------------------------------";
        "Block" iblock "of" nblocks;
        pause(1e-20);
    endif;

    /** set start time **/
    tstart = date;    

    phi      = zeros(nsim,nvar_*k);    
    sigmatrq = zeros(nsim,nvar_);
    sigmatr  = zeros(nsim,nvar_^2);

    j=1;
    do until j > nsim;

        {phi_new1,Sigmatr_new1,sigmatrq_new1}=paradraw(nobs,k,sigma_tilde,phi_tilde,XX);
        sigmatrq[j,.] = sigmatrq_new1;
        sigmatr[j,.]  = vec(Sigmatr_new1')';  
        phi[j,.]      = phi_new1';

        j=j+1;
        if _verbose;
            if fmod(j,nsim/10)==0;".";; pause(1e-20); endif;
        endif;

    endo;

    /** set finish time **/
    tfin = date;

    if _verbose;
        "";
        "Elapsed Time"; etstr(ethsec(tstart,tfin));
        "";
    endif;  /** verbose **/
    
    phi_mean     = phi_mean + sumc(phi)';
    sigmatr_mean = sigmatr_mean + sumc(sigmatr)';

    wr = writer(fhparavar,(phi~sigmatrq));

    iblock=iblock+1;

endo;

closeall;

/* getting the estimates for parameters and var-cov matrix of residuals */
phi_mean      = phi_mean/nsim_total;
sigmatr_mean  = sigmatr_mean/nsim_total;

PHI_hat      = reshape(phi_mean,nvar_,k)';
SIGMAtr_hat  = reshape(sigmatr_mean,nvar_,nvar_)';
SIGMA_hat    = SIGMAtr_hat'*SIGMAtr_hat;

if _verbose;

/** start log file **/
outstr = resupath $+ "\\para.log"; 
output file = ^outstr reset;

    "";
    "===================================";
    "Model            " $lmodel;
    "Prior            " $lprior;
    "Subsample        " $subTstr;
    "";
    "===================================";
    "";
    "";
    
   "PHI =";
    PHI_tilde;
    PHI_hat;
    "";

    "SIGMA =";
    SIGMA_tilde;
    SIGMA_hat;
    "";

    /** close log file **/
    output off;

endif;

end;
    
/***************************************************************************************************************************************************/


proc(3) = paradraw(nobs,k,sigma_tilde,phi_tilde,XX);
local phi_new, sigma_new, sigmatr_new, sigmatrq_new, q_new;

        /* Drawing from Sigma ~ IW((nobs+n_dummy)*sigma_tilde,v)
        */
        if nobsscale_ le 100;
           sigma_new   = iwish(nobs*nobsscale_-k,nobs*nobsscale_*sigma_tilde);
        else;
           sigma_new   = sigma_tilde;
        endif;
        sigmatr_new = chol(sigma_new)';
        
        /* Draw a unit length vector
        */
        q_new = rndn(nvar_,1);
        q_new = q_new./(sqrt(q_new'*q_new));
        
        /* Phi_eps 
        */
        sigmatrq_new = (Sigmatr_new*q_new)'; 
        
        /* Drawing from Phi|Sigma ~ N(phi_tilde, sigma_new.*.inv(XX'*XX))
        */
        if nobsscale_ le 100;
           phi_new = mrndn(vec(phi_tilde),sigma_new.*.inv(XX'*XX)/nobsscale_); 
        else;
           phi_new = vec(phi_tilde);
        endif;

retp(phi_new,sigmatr_new,sigmatrq_new);
endp;


/* This procedure samples from an inverted-Wishart distribution                                                     */

proc(1) = iwish(v,S);  
/* where v is the df and S is the location parameter defined as the sumof squared residuals ie (Y-X*beta)'*(Y-X*beta) */

local nvar, C, Z, IW, W;

nvar    = cols(S);
C       = cholpsd(inv(S))';
Z       = C*rndn(nvar,v);
W       = Z*Z';
IW      = inv(W);
retp(IW);
endp;


/***************************************************************************************************************************************************/



/* This procedure samples from a multivariate normal distribution with mean mu and var-cov matrix Sigma             *
/ NOTE that: this command can also be used for sampling from a non standard univariate normal                       */

proc(1) = mrndn(mu,Sigma);

local nvar, C, Z, i;

nvar    = cols(Sigma);
C       = cholpsd(Sigma);
Z       = mu+C*rndn(nvar,1);
retp(Z);
endp;

/***************************************************************************************************************************************************/

